// JavaScript Document

var wsApp;

// Helper functions
function getEventObj(e)
{
  return e || window.event;
}
function getEventTarget(e)
{
  var eventObj = getEventObj(e);
  return eventObj.target || eventObj.srcElement;
}

// Getters of special elements

function getClientElem()
{
  return document.getElementById('wsPlayer');
}

function getSidebarElem()
{
  return document.getElementById('sideBar');
}
function getMainClientElem()
{
  return document.getElementById('mainClient');
}

function getControllerElem()
{
  return document.getElementById('wsController');
}

//========== Size Control of SideBar and MainClient ===============

function maximizeSidebar()
{
  getSidebarElem().className = '';
  getMainClientElem().className = '';
  wsApp.setFixedSpaceWidth(260);
}
function minimizeSidebar()
{
  getSidebarElem().className = 'Minimize';
  getMainClientElem().className = 'Maximize';
  wsApp.setFixedSpaceWidth(110);
}

function toggleSidebarSize()
{
  if (getSidebarElem().className == '')
    minimizeSidebar();
  else
    maximizeSidebar();
}

function reactSidebarSizeTogglerClick(e)
{
  toggleSidebarSize();
  try { Event.stop(e); } catch(e){};
  return false;
}

//========== Tab Control of SideBar ===============
var sidebarTabPageMappings = {};
var classNameCurrTab = 'CurrTab';
var classNameTabPage = 'TabPage';

function fillDefaultSidebarTabPageMappings()
{
	sidebarTabPageMappings['tabSlides'] = 'wsSlideDir';
	sidebarTabPageMappings['tabInfo'] = 'wsAppInfo';
	sidebarTabPageMappings['tabOptions'] = 'wsOptions';
	sidebarTabPageMappings['tabLog'] = 'logPanel';
};
fillDefaultSidebarTabPageMappings();

function hideAllTabPages()
{
	for (var tabId in sidebarTabPageMappings)
	{
		var pageId = sidebarTabPageMappings[tabId];
		if (typeof(pageId) == 'string')
		{
			var elem = document.getElementById(pageId);
			if (elem)
			{
				try
				{
					Element.removeClassName(elem, classNameCurrTab);
				}
				catch(e)
				{
					elem.className = classNameTabPage;
				}
			}
		}
	}
	// hide slide list scroller buttons also
	//Element.hide($('slideListScrollButtonPanel'));
	document.getElementById('slideListScrollButtonPanel').style.display = 'none';
}
function showTabPage(pageId)
{
	var elem = document.getElementById(pageId);
	if (elem)
	{
		try
		{
			Element.addClassName(elem, classNameCurrTab);
			if (pageId == 'wsSlideDir')  // show slide scroller button also
			  //Element.show($('slideListScrollButtonPanel'));
			  document.getElementById('slideListScrollButtonPanel').style.display = '';
		}
		catch(e)
		{
			elem.className = classNameTabPage + ' ' + classNameCurrTab;
			if (pageId == 'wsSlideDir')  // show slide scroller button also
			  document.getElementById('slideListScrollButtonPanel').style.display = '';
		}
	}
}

function blurAllTabTabs()
{
	for (var tabId in sidebarTabPageMappings)
	{
		if (typeof(tabId) == 'string')
		{
			var elem = document.getElementById(tabId);
			if (elem)
			{
				elem.className = '';
			}
		}
	}
}
function focusTabTab(tabId)
{
  var elem = document.getElementById(tabId);
  if (elem)
    elem.className = classNameCurrTab;
}

function gotoTab(tabId)
{
  var pageId = sidebarTabPageMappings[tabId];
  hideAllTabPages();
  blurAllTabTabs();
  focusTabTab(tabId);
  showTabPage(pageId);
}

function reactTabTabClick(e)
{
  var targetTab = getEventTarget(e);
  if (targetTab.tagName.toLowerCase() == 'a')
  {
    if (targetTab.id)
      gotoTab(targetTab.id);
  }
  try { Event.stop(e); } catch(e){};
  return false;
}

//========== Tab: Info ===============
function getInfoTableElem()
{
  return document.getElementById('infoTableBody');
}

function clearTableRows(root)
{
  var rows = root.getElementsByTagName('tr');
  for (var i = rows.length - 1; i >= 0; --i)
    rows[i].parentNode.removeChild(rows[i]);
}

function addInfoRow(key, value)
{
  var root = getInfoTableElem();
  var row = document.createElement('tr');
  // key
  var cell = document.createElement('td');
  var cellText = document.createTextNode(key);
  cell.appendChild(cellText);
  row.appendChild(cell);
  // value
  var cell = document.createElement('td');
  var cellText = document.createTextNode(value);
  cell.appendChild(cellText);
  row.appendChild(cell);
  root.appendChild(row);
}

function fillAppInfo(wsApp)
{
  clearTableRows(getInfoTableElem());
  var metas = wsApp.getMetaData();
  var keys = metas.getKeys();
  var value;
  for (var i = 0, l = keys.length; i < l; ++i)
  {
    value = metas.get(keys[i]);
    if (value)
      addInfoRow(keys[i], value);
  }
  // WebShow version info
  try
  {
    addInfoRow('WebShow Version', WebShow.Version);
  }
  catch(e)
  {
    
  }
}

function prepareAutoZoom(wsApp)
{
  wsApp.setAutoZoomerUsePredefinedRatio(true);
  wsApp.setAutoZoomWidthRatio(1);
  wsApp.setAutoZoomHeightRatio(1);
  wsApp.setFixedSpaceWidth(260);  // sidebar width width gap
  wsApp.setFixedSpaceHeight(60);  // controller height plus mainclient padding
}

//========== Tab: Options ===============
function getControlValue(ctrlElem)
{
	if (ctrlElem)
    return ctrlElem.value;
  else
  	return null;
}
function setControlValue(ctrlElem, value)
{
	if (ctrlElem)
    ctrlElem.value = value;
}
function getControlChecked(ctrlElem)
{
	if (ctrlElem)
    return ctrlElem.checked;
  else
  	return null;
}
function setControlChecked(ctrlElem, value)
{
	if (ctrlElem)
    ctrlElem.checked = value;
}

function loadWsAppOptions(app)
{
  // Auto play interval
  setControlValue(document.getElementById('edtAutoPlayInterval'),
    (app.getActualAutoPlayInterval() || 0) / 1000);
  // Play hidden slides
  setControlChecked(document.getElementById('checkboxPlayHidden'),
    !!app.getForcePlayHiddenSlides());  
  // Instantly Operations
  setControlChecked(document.getElementById('checkBoxInstantlyOperations'),
    !!app.getOperationInstantly())
  // Loop play
  setControlChecked(document.getElementById('checkBoxLoopPlay'),
    !!app.getLoopPlay());
  // auto zoom
  setControlChecked(document.getElementById('checkBoxAutoZoom'),
    !!app.getAutoZoomerEnabled())
}
function applyWsAppOptions(app)
{
  // Auto play interval
  app.setAutoPlayInterval(parseFloat(getControlValue(document.getElementById('edtAutoPlayInterval'))) * 1000);
  // Play hidden slides
  app.setForcePlayHiddenSlides(!!getControlChecked(document.getElementById('checkboxPlayHidden')));
  // Instantly Operations
  app.setOperationInstantly(!!getControlChecked(document.getElementById('checkBoxInstantlyOperations')));
  // Loop play
  app.setLoopPlay(!!getControlChecked(document.getElementById('checkBoxLoopPlay')));  
  // auto zoom
  app.setAutoZoomerEnabled(!!getControlChecked(document.getElementById('checkBoxAutoZoom')));  
}

function loadPlayerOptions()
{
  var index;
  // Sidebar position
  index = getClientElem().className == ''? 1: 0;
  document.getElementById('listSidebarPos').selectedIndex = index;
  // Controller position
  var className = getControllerElem().className;
  index = className.indexOf('Inside') >= 0? 0: 1;
  document.getElementById('listControllerPos').selectedIndex = index;
  // Auto hide Controller
  document.getElementById('checkBoxAutoHideController').checked = className.indexOf('AutoHideCtrl') >= 0;
}
function applyPlayerOptions()
{
  var index;
  // Sidebar position
  index = document.getElementById('listSidebarPos').selectedIndex;
  if (index == 1)
    getClientElem().className = ''
  else
    getClientElem().className = 'LayoutSidebarLeft';
  // Controller position and Autohide
  index = document.getElementById('listControllerPos').selectedIndex;
  var className = '';
  if (index == 1)
    className = '';
  else
    className = 'Inside ';
  if (document.getElementById('checkBoxAutoHideController').checked)
    className = className + 'AutoHideCtrl';
  getControllerElem().className = className;
}

function loadOptions()
{
  loadPlayerOptions();
  try
  {
    if (WebShow.app)
      loadWsAppOptions(WebShow.app);
  }
  catch(e) {}
}
function applyOptions()
{
  applyPlayerOptions();
  try
  {
    if (WebShow.app)
      applyWsAppOptions(WebShow.app);
  }
  catch(e)
  {
    
  }
}

function reactBtnApplyOptionsClick(e)
{
  applyOptions();
  try { Event.stop(e); } catch(e){};
  return false;
}
function reactBtnCancelOptionsClick(e)
{
  loadOptions();
  try { Event.stop(e); } catch(e){};
  return false;
}

// --- Slidelist scroll -------------------------------------
var defaultStep = 6;
var scrollStepMaxCount = 0;
var scrollStepCount = 0;
var step = defaultStep;
var scrollTimer;
function scrollElementDown(elemId)
{
  scrollStepCount++;
  if (scrollStepMaxCount && (scrollStepCount < scrollStepMaxCount))
  {
    document.getElementById(elemId).scrollTop+=step;
    scrollTimer = setTimeout('scrollElementDown("'+ elemId + '")', 10);
  }
}

function scrollElementUp(elemId)
{
  scrollStepCount++;
  if (scrollStepMaxCount && (scrollStepCount < scrollStepMaxCount))
  {
    document.getElementById(elemId).scrollTop-=step;
    scrollTimer = setTimeout('scrollElementUp("' + elemId + '")', 10);
  }
}
function scrollElementToTop(elemId)
{
  document.getElementById(elemId).scrollTop = 0;
}
function scrollElementToBottom(elemId)
{
  document.getElementById(elemId).scrollTop = document.getElementById(elemId).scrollHeight;
}

function scrollSlideListToTop(e)
{
  scrollElementToTop('sidePanelScroller');
}
function scrollSlideListToBottom(e)
{
  scrollElementToBottom('sidePanelScroller');
}
function scrollPageUp(e)
{
  stopScrollSlideList();
  scrollStepMaxCount = 20;
  scrollElementUp('wsSlideDirClient');
  /*
  var elem = document.getElementById('wsSlideDirClient');
  elem.scrollTop -= 75;
  */
  try { Event.stop(e); } catch(e){};
  return false;
}
function scrollPageDown(e)
{
  stopScrollSlideList();
  scrollStepMaxCount = 20;
  scrollElementDown('wsSlideDirClient');
  /*
  var elem = document.getElementById('wsSlideDirClient');
  elem.scrollTop += 75;
  */
  try { Event.stop(e); } catch(e){};
  return false;
}
function startScrollUpSlideList(e)
{
  //alert(document.getElementById('sidePanelScroller'));
  stopScrollSlideList();
  scrollElementUp('sidePanelScroller');
  try { Event.stop(e); } catch(e){};
  return false;
}
function startScrollDownSlideList(e)
{
  stopScrollSlideList();
  scrollElementDown('sidePanelScroller');
  try { Event.stop(e); } catch(e){};
  return false;
}
function stopScrollSlideList(e)
{
  if (scrollTimer)
    clearTimeout(scrollTimer);
  scrollStepCount = 0;
  try { Event.stop(e); } catch(e){};
  return false;
}

// iscroll on ipad or iphone
var iscrollerSlideList = null;
var iscrollerLog = null;
function initIScroll()
{
  setTimeout(function()
    { 
      if (window.iScroll)
      {
        iscrollerSlideList = new iScroll(document.getElementById('wsSlideDir'));
        iscrollerSlideLog = new iScroll(document.getElementById('wsLog'));
      }
    },
    100);
}
function refreshIScroll()
{
  if (iscrollerSlideList)
  {
    iscrollerSlideList.refresh();
  }
  if (iscrollerLog)
  {
    iscrollerLog.refresh();
  }
}


function preventHref(e)
{
  try { Event.stop(e); } catch(e){};
  return false;
}

//========== Init ===============
function init()
{
  fillDefaultSidebarTabPageMappings();
  try  // avoid IE raise exception while designing
  {
    //if (Event && Event.observe)
    {
      Event.observe('sidebarSizeToggler', 'click', reactSidebarSizeTogglerClick);
      Event.observe('sideBarTabs', 'click', reactTabTabClick);
      Event.observe('btnApplyOptions', 'click', reactBtnApplyOptionsClick);
      Event.observe('btnCancelOptions', 'click', reactBtnCancelOptionsClick);
      
      Event.observe('btnSlideListScrollToTop', 'click', scrollSlideListToTop);
      Event.observe('btnSlideListScrollToBottom', 'click', scrollSlideListToBottom);
      Event.observe('btnSlideListScrollUp', 'click', scrollPageUp);
      Event.observe('btnSlideListScrollDown', 'click', scrollPageDown);
      /*
      Event.observe('btnSlideListScrollUp', 'click', preventHref);
      Event.observe('btnSlideListScrollUp', 'mousedown', startScrollUpSlideList);
      Event.observe('btnSlideListScrollUp', 'mouseup', stopScrollSlideList);
      Event.observe('btnSlideListScrollDown', 'click', preventHref);
      Event.observe('btnSlideListScrollDown', 'mousedown', startScrollDownSlideList);
      Event.observe('btnSlideListScrollDown', 'mouseup', stopScrollSlideList);
      */
      
      Event.observe(document.documentElement, WebShow.AppLauncher.APP_LAUNCH_EVENT, reactWsAppLoaded);
      
      //installSlideNavEventHandlers();
    }
  }
  catch(e)
  {
    document.getElementById('sidebarSizeToggler').onclick = reactSidebarSizeTogglerClick;
    document.getElementById('sideBarTabs').onclick = reactTabTabClick;
    document.getElementById('btnApplyOptions').onclick = reactBtnApplyOptionsClick;
    document.getElementById('btnCancelOptions').onclick = reactBtnCancelOptionsClick;
    
    document.getElementById('btnSlideListScrollToTop').onclick = scrollSlideListToTop;
    document.getElementById('btnSlideListScrollToBottom').onclick = scrollSlideListToBottom;
    document.getElementById('btnSlideListScrollUp').onclick = scrollPageUp;
    document.getElementById('btnSlideListScrollDown').onclick = scrollPageDown;
    /*
    document.getElementById('btnSlideListScrollUp').onclick = preventHref;
    document.getElementById('btnSlideListScrollUp').onmousedown = startScrollUpSlideList;
    document.getElementById('btnSlideListScrollUp').onmouseup = stopScrollSlideList;
    document.getElementById('btnSlideListScrollDown').onclick = preventHref;
    document.getElementById('btnSlideListScrollDown').onmousedown = startScrollDownSlideList;
    document.getElementById('btnSlideListScrollDown').onmouseup = stopScrollSlideList;
    */
    loadOptions();
  }
  try
  {
    if (WebShow.Logger)
    {
      WebShow.Logger.element = document.getElementById('wsLog');
      //WebShow.Logger.onError = loggerOnError;
    }
    if (WebShow.app)  // already loaded
      reactWsAppLoaded();
  }
  catch(e) {  }
}

function reactWsAppLoaded(e)
{
  if (WebShow.app)
  {
    wsApp = WebShow.app;
    wsApp.onAppReady = reactOnWsAppReady;
    fillAppInfo(WebShow.app);
    prepareAutoZoom(WebShow.app);
    initIScroll();
    loadOptions();
  }
}

function reactOnWsAppReady(app)
{
  refreshIScroll();
}

try
{
  Event.observe(document, 'dom:loaded', init);
}
catch(e)
{  
  window.onload = init;
}